import { __decorate, __metadata } from 'tslib';
import { ɵɵdefineInjectable, Injectable, ElementRef, Renderer2, Input, Directive, ɵɵinject, NgModule } from '@angular/core';
import { Observable, throwError } from 'rxjs';
import { share, tap, catchError } from 'rxjs/operators';
import { CommonModule } from '@angular/common';
import { HttpResponse, HttpErrorResponse, HttpClientModule, HTTP_INTERCEPTORS } from '@angular/common/http';

let CounterService = class CounterService {
    constructor() {
        this._connectionCounter = 0;
        this._observable = Observable.create(subscriber => {
            this._subscriber = subscriber;
        }).pipe(share());
    }
    get observable() {
        return this._observable;
    }
    increment() {
        this._connectionCounter += 1;
        this._next();
    }
    decrement() {
        this._connectionCounter -= 1;
        this._next();
    }
    _next() {
        if (this._subscriber) {
            this._subscriber.next(this._connectionCounter);
        }
    }
};
CounterService.ɵprov = ɵɵdefineInjectable({ factory: function CounterService_Factory() { return new CounterService(); }, token: CounterService, providedIn: "root" });
CounterService = __decorate([
    Injectable({
        providedIn: 'root'
    }),
    __metadata("design:paramtypes", [])
], CounterService);

let HttpClientBusyDirective = class HttpClientBusyDirective {
    constructor(el, renderer, counter) {
        this.el = el;
        this.renderer = renderer;
        this.counter = counter;
        this.httpClientBusyConfig = {};
    }
    _updateStatus(count) {
        const busy = count > 0;
        const hasIdle = this.el.nativeElement.classList.contains(this.httpClientBusyConfig.idle);
        const hasBusy = this.el.nativeElement.classList.contains(this.httpClientBusyConfig.busy);
        if (busy) {
            if (this.httpClientBusyConfig.idle && hasIdle) {
                this.renderer.removeClass(this.el.nativeElement, this.httpClientBusyConfig.idle);
            }
            if (this.httpClientBusyConfig.busy && !hasBusy) {
                this.renderer.addClass(this.el.nativeElement, this.httpClientBusyConfig.busy);
            }
        }
        else {
            if (this.httpClientBusyConfig.busy && hasBusy) {
                this.renderer.removeClass(this.el.nativeElement, this.httpClientBusyConfig.busy);
            }
            if (this.httpClientBusyConfig.idle && !hasIdle) {
                this.renderer.addClass(this.el.nativeElement, this.httpClientBusyConfig.idle);
            }
        }
    }
    ngOnInit() {
        this.httpClientBusyConfig.idle = this.httpClientBusy.idle || 'fade';
        this.httpClientBusyConfig.busy = this.httpClientBusy.busy || '';
        this._updateStatus(0);
        this.autoUnsubscribe = this.counter.observable.subscribe(next => this._updateStatus(next));
    }
    ngOnDestroy() {
        this.autoUnsubscribe.unsubscribe();
    }
};
HttpClientBusyDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 },
    { type: CounterService }
];
__decorate([
    Input(),
    __metadata("design:type", Object)
], HttpClientBusyDirective.prototype, "httpClientBusy", void 0);
HttpClientBusyDirective = __decorate([
    Directive({
        selector: '[httpClientBusy]'
    }),
    __metadata("design:paramtypes", [ElementRef, Renderer2, CounterService])
], HttpClientBusyDirective);

let HttpClientBusyInterceptor = class HttpClientBusyInterceptor {
    constructor(counter) {
        this.counter = counter;
    }
    intercept(req, next) {
        this.counter.increment();
        return next.handle(req).pipe(tap(evt => {
            if (evt instanceof HttpResponse) {
                this.counter.decrement();
            }
        }), catchError(err => {
            if (err instanceof HttpErrorResponse) {
                this.counter.decrement();
            }
            return throwError(err);
        }));
    }
};
HttpClientBusyInterceptor.ctorParameters = () => [
    { type: CounterService }
];
HttpClientBusyInterceptor.ɵprov = ɵɵdefineInjectable({ factory: function HttpClientBusyInterceptor_Factory() { return new HttpClientBusyInterceptor(ɵɵinject(CounterService)); }, token: HttpClientBusyInterceptor, providedIn: "root" });
HttpClientBusyInterceptor = __decorate([
    Injectable({
        providedIn: 'root'
    }),
    __metadata("design:paramtypes", [CounterService])
], HttpClientBusyInterceptor);

var HttpClientBusyModule_1;
let HttpClientBusyModule = HttpClientBusyModule_1 = class HttpClientBusyModule {
    static forRoot() {
        return {
            ngModule: HttpClientBusyModule_1
        };
    }
};
HttpClientBusyModule = HttpClientBusyModule_1 = __decorate([
    NgModule({
        imports: [CommonModule, HttpClientModule],
        declarations: [HttpClientBusyDirective],
        exports: [HttpClientBusyDirective],
        providers: [{ provide: HTTP_INTERCEPTORS, useClass: HttpClientBusyInterceptor, multi: true }]
    })
], HttpClientBusyModule);

/*
 * Public API Surface of ngx-httpclient-busy
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CounterService, HttpClientBusyDirective, HttpClientBusyModule, HttpClientBusyInterceptor as ɵa };
//# sourceMappingURL=ngx-httpclient-busy.js.map
