import { __decorate, __metadata } from 'tslib';
import { ɵɵdefineInjectable, Injectable, ElementRef, Renderer2, Input, Directive, ɵɵinject, NgModule } from '@angular/core';
import { Observable, throwError } from 'rxjs';
import { share, tap, catchError } from 'rxjs/operators';
import { CommonModule } from '@angular/common';
import { HttpResponse, HttpErrorResponse, HttpClientModule, HTTP_INTERCEPTORS } from '@angular/common/http';

var CounterService = /** @class */ (function () {
    function CounterService() {
        var _this = this;
        this._connectionCounter = 0;
        this._observable = Observable.create(function (subscriber) {
            _this._subscriber = subscriber;
        }).pipe(share());
    }
    Object.defineProperty(CounterService.prototype, "observable", {
        get: function () {
            return this._observable;
        },
        enumerable: true,
        configurable: true
    });
    CounterService.prototype.increment = function () {
        this._connectionCounter += 1;
        this._next();
    };
    CounterService.prototype.decrement = function () {
        this._connectionCounter -= 1;
        this._next();
    };
    CounterService.prototype._next = function () {
        if (this._subscriber) {
            this._subscriber.next(this._connectionCounter);
        }
    };
    CounterService.ɵprov = ɵɵdefineInjectable({ factory: function CounterService_Factory() { return new CounterService(); }, token: CounterService, providedIn: "root" });
    CounterService = __decorate([
        Injectable({
            providedIn: 'root'
        }),
        __metadata("design:paramtypes", [])
    ], CounterService);
    return CounterService;
}());

var HttpClientBusyDirective = /** @class */ (function () {
    function HttpClientBusyDirective(el, renderer, counter) {
        this.el = el;
        this.renderer = renderer;
        this.counter = counter;
        this.httpClientBusyConfig = {};
    }
    HttpClientBusyDirective.prototype._updateStatus = function (count) {
        var busy = count > 0;
        var hasIdle = this.el.nativeElement.classList.contains(this.httpClientBusyConfig.idle);
        var hasBusy = this.el.nativeElement.classList.contains(this.httpClientBusyConfig.busy);
        if (busy) {
            if (this.httpClientBusyConfig.idle && hasIdle) {
                this.renderer.removeClass(this.el.nativeElement, this.httpClientBusyConfig.idle);
            }
            if (this.httpClientBusyConfig.busy && !hasBusy) {
                this.renderer.addClass(this.el.nativeElement, this.httpClientBusyConfig.busy);
            }
        }
        else {
            if (this.httpClientBusyConfig.busy && hasBusy) {
                this.renderer.removeClass(this.el.nativeElement, this.httpClientBusyConfig.busy);
            }
            if (this.httpClientBusyConfig.idle && !hasIdle) {
                this.renderer.addClass(this.el.nativeElement, this.httpClientBusyConfig.idle);
            }
        }
    };
    HttpClientBusyDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.httpClientBusyConfig.idle = this.httpClientBusy.idle || 'fade';
        this.httpClientBusyConfig.busy = this.httpClientBusy.busy || '';
        this._updateStatus(0);
        this.autoUnsubscribe = this.counter.observable.subscribe(function (next) { return _this._updateStatus(next); });
    };
    HttpClientBusyDirective.prototype.ngOnDestroy = function () {
        this.autoUnsubscribe.unsubscribe();
    };
    HttpClientBusyDirective.ctorParameters = function () { return [
        { type: ElementRef },
        { type: Renderer2 },
        { type: CounterService }
    ]; };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], HttpClientBusyDirective.prototype, "httpClientBusy", void 0);
    HttpClientBusyDirective = __decorate([
        Directive({
            selector: '[httpClientBusy]'
        }),
        __metadata("design:paramtypes", [ElementRef, Renderer2, CounterService])
    ], HttpClientBusyDirective);
    return HttpClientBusyDirective;
}());

var HttpClientBusyInterceptor = /** @class */ (function () {
    function HttpClientBusyInterceptor(counter) {
        this.counter = counter;
    }
    HttpClientBusyInterceptor.prototype.intercept = function (req, next) {
        var _this = this;
        this.counter.increment();
        return next.handle(req).pipe(tap(function (evt) {
            if (evt instanceof HttpResponse) {
                _this.counter.decrement();
            }
        }), catchError(function (err) {
            if (err instanceof HttpErrorResponse) {
                _this.counter.decrement();
            }
            return throwError(err);
        }));
    };
    HttpClientBusyInterceptor.ctorParameters = function () { return [
        { type: CounterService }
    ]; };
    HttpClientBusyInterceptor.ɵprov = ɵɵdefineInjectable({ factory: function HttpClientBusyInterceptor_Factory() { return new HttpClientBusyInterceptor(ɵɵinject(CounterService)); }, token: HttpClientBusyInterceptor, providedIn: "root" });
    HttpClientBusyInterceptor = __decorate([
        Injectable({
            providedIn: 'root'
        }),
        __metadata("design:paramtypes", [CounterService])
    ], HttpClientBusyInterceptor);
    return HttpClientBusyInterceptor;
}());

var HttpClientBusyModule = /** @class */ (function () {
    function HttpClientBusyModule() {
    }
    HttpClientBusyModule_1 = HttpClientBusyModule;
    HttpClientBusyModule.forRoot = function () {
        return {
            ngModule: HttpClientBusyModule_1
        };
    };
    var HttpClientBusyModule_1;
    HttpClientBusyModule = HttpClientBusyModule_1 = __decorate([
        NgModule({
            imports: [CommonModule, HttpClientModule],
            declarations: [HttpClientBusyDirective],
            exports: [HttpClientBusyDirective],
            providers: [{ provide: HTTP_INTERCEPTORS, useClass: HttpClientBusyInterceptor, multi: true }]
        })
    ], HttpClientBusyModule);
    return HttpClientBusyModule;
}());

/*
 * Public API Surface of ngx-httpclient-busy
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CounterService, HttpClientBusyDirective, HttpClientBusyModule, HttpClientBusyInterceptor as ɵa };
//# sourceMappingURL=ngx-httpclient-busy.js.map
